package com.ruoyi.framework.mybatis;

import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.ReflectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.annotation.FieldFill;
import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableName;
import com.baomidou.mybatisplus.extension.service.IService;
import org.springframework.beans.BeansException;
import org.springframework.beans.factory.config.BeanPostProcessor;
import org.springframework.context.annotation.Configuration;

import java.lang.reflect.Field;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

@Configuration
public class RemoveParam implements BeanPostProcessor {

    public static final Map<String, Boolean> updateUserMap = new ConcurrentHashMap<>();
    public static final Map<String, Boolean> updateTimeMap = new ConcurrentHashMap<>();

    @Override
    public Object postProcessAfterInitialization(Object bean, String beanName) throws BeansException {

        if (bean instanceof IService) {
            Class<?> entityClass = ((IService<?>) bean).getEntityClass();
            TableName annotation = entityClass.getAnnotation(TableName.class);
            String tableName;
            if (annotation != null && StrUtil.isNotBlank(annotation.value())) {
                tableName = annotation.value();
            } else {
                tableName = StrUtil.toUnderlineCase(entityClass.getSimpleName());
            }
            Field[] fields = ReflectUtil.getFields(entityClass);
            for (Field field : fields) {
                TableField tableField = field.getAnnotation(TableField.class);
                if (tableField == null) {
                    continue;
                }
                FieldFill fill = tableField.fill();
                if (ObjectUtil.notEqual(fill, FieldFill.INSERT_UPDATE) && ObjectUtil.notEqual(fill, FieldFill.UPDATE)) {
                    continue;
                }
                if ("update_time".equals(tableField.value()) || "updateTime".equals(field.getName())) {
                    RemoveParam.updateTimeMap.put(tableName, true);
                } else if ("update_user".equals(tableField.value()) || "updateUser".equals(field.getName())) {
                    RemoveParam.updateUserMap.put(tableName, true);
                }
            }
        }

        return bean;
    }

}
